@@ -63,7 +63,8 @@ OpStatus = IntEnum('OpStatus', 'INIT \
                                 FAILED \
                                 FAILED_UNRECOVERABLE \
                                 FILE_NOT_FOUND \
-                                FINISHED')
+                                FINISHED \
+                                FINISHED_WARNING')
 
 OpCommand = IntEnum('OpCommand', 'START_TRANSFER \
                                   UPDATE_PROGRESS \
@@ -75,6 +76,11 @@ OpCommand = IntEnum('OpCommand', 'START_TRANSFER \
                                   STOP_TRANSFER_BY_RECEIVER \
                                   REMOVE_TRANSFER')
 
+class ReceiveError(Exception):
+    def __init__(self, message, fatal=True):
+        self.fatal = fatal
+        super().__init__(message)
+
 class InterfaceInfo():
     def __init__(self, ip4, ip6, iface=None):
         self.iface = iface
@@ -255,6 +261,8 @@ def have_free_space(size):
     if free == 0:
         return True
 
+    free -= prefs.get_min_free_space() * 1024
+
     logging.debug("need: %s, have %s" % (GLib.format_size(size), GLib.format_size(free)))
 
     return size < free
@@ -263,7 +271,8 @@ def files_exist(base_names):
     for name in base_names:
         path = os.path.join(prefs.get_save_path(), name)
         logging.debug("(server side) Checking if file or folder %s already exists." % (path,))
-        if GLib.file_test(path, GLib.FileTest.EXISTS):
+        file = Gio.File.new_for_path(path)
+        if file.query_exists(None):
             return True
 
     return False
@@ -428,6 +437,7 @@ class AboutDialog():
                                  icon_name="org.x.Warpinator",
                                  logo_icon_name="org.x.Warpinator",
                                  comments=_("Send and Receive Files across the Network"),
+                                 website="https://github.com/linuxmint/warpinator",
                                  version=config.VERSION,
                                  license_type=Gtk.License.GPL_3_0)
 
